--------------------------------------------------------------------------------
-- Firework Rocket by Vectar666
-- Based on:
-- Weapon Mine + Projectile Mine
-- Projectile Rocket
-- Original Carnage Contest Weapons
-- Script by DC, March 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.v666==nil then cc.v666={} end
cc.v666.firework={}
cc.v666.firework.firework={}
cc.v666.firework.rocket={}

-- Load & Prepare Ressources
cc.v666.firework.gfx_wpn=loadgfx("weapons/firework.bmp")					-- Weapon Image
setmidhandle(cc.v666.firework.gfx_wpn)
cc.v666.firework.gfx_pro=loadgfx("weapons/firework_rocket.bmp")
setmidhandle(cc.v666.firework.gfx_pro)
cc.v666.firework.sfx_attack=loadsfx("throw.ogg")							-- Attack Sound
cc.v666.firework.sfx_bounce=loadsfx("bounce.wav")							-- Bounce Sound
cc.v666.firework.sfx_flyaway=loadsfx("firework.wav")

--------------------------------------------------------------------------------
-- Weapon: mine
--------------------------------------------------------------------------------

cc.v666.firework.id=addweapon("cc.v666.firework","Firework",cc.v666.firework.gfx_pro,1)				-- Add Weapon (1 use)

function cc.v666.firework.draw()													-- Draw
	if getplayeraction(0)==0 and weapon_shots<=0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setscale(1,1)
		setrotation(0)
		drawimage(cc.v666.firework.gfx_wpn,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)+2)
	end
end

function cc.v666.firework.attack(attack)											-- Attack
	if (weapon_shots<=0) then
		if attack==1 then
			-- No more weapon switching!
			useweapon(1)
			playsound(cc.v666.firework.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.v666.firework.firework.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*7
			projectiles[id].y=getplayery(0)
			-- Set speed of projectile
			projectiles[id].sx=getplayerdirection(0)*0.5
			projectiles[id].sy=-1
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: firework
--------------------------------------------------------------------------------

cc.v666.firework.firework.id=addprojectile("cc.v666.firework.firework")	-- Add Projectile

function cc.v666.firework.firework.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	-- Draw projectile
	drawimage(cc.v666.firework.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.v666.firework.firework.update(id)					-- Update
	-- Gravity influence on speed + decrease x speed
	projectiles[id].sx=projectiles[id].sx*0.95
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(cc.v666.firework.gfx_wpn,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(cc.v666.firework.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.05
				msubx=-msubx*0.05
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(cc.v666.firework.gfx_wpn,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(cc.v666.firework.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
			end
		else
			projectiles[id].ignore=0
		end		
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	if getframesleft()==0 then
	--shoot
			iid=createprojectile(cc.v666.firework.rocket.id)
			projectiles[iid]={}
			-- Set initial position of projectile
			projectiles[iid].x=projectiles[id].x
			projectiles[iid].y=projectiles[id].y-10
			-- Initial movement
			projectiles[iid].sx=0
			projectiles[iid].sy=-10.0
			-- Set speed of projectile
			projectiles[iid].sx=0
			projectiles[iid].sy=-10.0
			-- Set time4
			projectiles[iid].timer=60
			freeprojectile(id)
	playsound(cc.v666.firework.sfx_flyaway)
	end
end


--projectile rocket
cc.v666.firework.rocket.id=addprojectile("cc.v666.firework.rocket")		-- Add Projectile

function cc.v666.firework.rocket.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.v666.firework.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.v666.firework.rocket.update(id)						-- Update
	-- Move
	cc.v666.firework.rocket.move(id,1)
end

function cc.v666.firework.rocket.move(id,fx)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	projectiles[id].timer=projectiles[id].timer-1
	-- Particle Tail
	if (fx==1) then
		particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
		particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
		particlefadealpha(0.01)
		particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(rot))*6,projectiles[id].y+math.cos(math.rad(rot))*6)
		particlefadealpha(0.04)
	end
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()/10
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col4x4,projectiles[id].x+math.sin(math.rad(rot))*3,projectiles[id].y-math.cos(math.rad(rot))*3)==1 or projectiles[id].timer==0 then
				-- Destroy terrain
				terrainexplosion(projectiles[id].x,projectiles[id].y,10,1)
				-- Crater
				grey=math.random(0,40)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater50,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater25,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				end
				-- Free projectile
				freeprojectile(id)
				local rx,ry
				for i=1,7 do
					randomseed(id*2011+12345*i+getframe()*666)
					createobject(o_fire,projectiles[id].x+random(-40,40),projectiles[id].y+random(-35,35))
					scroll(projectiles[id].x,projectiles[id].y)
					for j=1,5 do
						randomseed(j*2011+id*12345*i+getframe()*666)
						rx=random(-45,45)
						ry=random(-45,45)
						particle(p_spark,projectiles[id].x+rx,projectiles[id].y+ry)
						particlespeed(rx/20,ry/20)
						particlesize(3,3)
						particlefadealpha(0.015)
						particlecolor(random(30,255),random(30,255),random(30,255))
					end
				end
				return 1
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end