--------------------------------------------------------------------------------
-- Snowball by Vectar666
-- Based on:
-- Weapon Spear + Projectile Spear
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.v666==nil then cc.v666={} end
cc.v666.snowball={}
cc.v666.snowball.snowball={}

-- Load & Prepare Ressources
cc.v666.snowball.gfx_wpn=loadgfx("weapons/snowball.bmp")				-- Weapon Image
setmidhandle(cc.v666.snowball.gfx_wpn)
cc.v666.snowball.sfx_attack=loadsfx("throw.ogg")				-- Attack Sound
cc.v666.snowball.sfx_impact=loadsfx("snowballhit.wav")				-- Impact Sound

--------------------------------------------------------------------------------
-- Weapon: Spear
--------------------------------------------------------------------------------

cc.v666.snowball.id=addweapon("cc.v666.snowball","Snowball",cc.v666.snowball.gfx_wpn,2)	-- Add Weapon (2 uses)
cc.v666.snowball.ammo=3													-- 1 Spear

function cc.v666.snowball.draw()										-- Draw
	if cc.v666.snowball.ammo-weapon_shots>0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(cc.v666.snowball.gfx_wpn,6,0)
	end
	if weapon_charge>0 and weapon_shots<3 then
		hudchargebar(weapon_charge,100)
	end
	-- HUD Crosshair
	if cc.v666.snowball.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
		hudammobar(3-weapon_shots,3)
	end
end

function cc.v666.snowball.attack(attack)								-- Attack
	if weapon_shots<3 then
		-- Decrement timer
		if weapon_timer>0 then
			weapon_timer=weapon_timer-1
		end
		-- Attack
		if (attack==1) and weapon_timer==0 then
			weapon_charge=weapon_charge+1							-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			weapon_timer=40
			playsound(cc.v666.snowball.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.v666.snowball.snowball.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))
			projectiles[id].y=getplayery(0)+3
			-- Initial collision check (avoid throwing into other objects)
			for i=0,10,1 do
				if collision(col5x5,projectiles[id].x+math.sin(math.rad(getplayerrotation(0)))*i,projectiles[id].y-math.cos(math.rad(getplayerrotation(0)))*i)==1 then
					if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
						if (i==0) then
							projectiles[id].x=getplayerx(0)
							projectiles[id].y=getplayery(0)+3
							break
						else
							projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*(i-1)
							projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*(i-1)
							break
						end
					end
				end
			end
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/75.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/75.0)*10.0
			weapon_charge=0
			if weapon_shots==3 then
				endturn()
			end
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Spear
--------------------------------------------------------------------------------

cc.v666.snowball.snowball.id=addprojectile("cc.v666.snowball.snowball")		-- Add Projectile

function cc.v666.snowball.snowball.draw(id)						-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.v666.snowball.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.v666.snowball.snowball.update(id)						-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.1
	projectiles[id].sy=projectiles[id].sy+getgravity()*1.5
	-- Move
	cc.v666.snowball.snowball.move(id)
end

function cc.v666.snowball.snowball.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*13,projectiles[id].y-math.cos(math.rad(rot))*13)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if playercollision()~=0 then
					-- Cause Player damage
					playerpush(playercollision(),projectiles[id].sx/10.0,projectiles[id].sy/10.0)
					playerdamage(playercollision(),10)
				elseif objectcollision()>0 then
					-- Cause Object Damage
					objectdamage(objectcollision(),10)
				end
				-- Effects
				playsound(cc.v666.snowball.sfx_impact)
				particle(p_bigblood,projectiles[id].x+math.sin(math.rad(rot))*14,projectiles[id].y-math.cos(math.rad(rot))*14)
				particlecolor(255,255,255)
				particlefadealpha(0.006)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end